// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package company

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"property-company/consts"
	"property-company/errors"
	dbmodel "property-company/model"
	pb_v1 "property-company/pb/v1"
	"property-company/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkRegisterParam(req *pb_v1.RegisterRequest) error {
	switch {
	case req.Username == "" || req.Password == "":
		return status.Error(10003, "账户和密码不能为空")
	case req.SocialCode == "":
		return status.Error(10003, "社会信用码不能为空")
	case req.CompanyName == "":
		return status.Error(10003, "公司名称不能为空")
	case req.LicenseType != 1 && req.LicenseType != 2:
		return status.Error(10003, "不支持的营业执照类型")
	case req.Expire == 0:
		req.Expire = -1
		//return status.Error(10003, "小区有效期不能为空")
		// TODO 暂时不作限制
	}
	return nil
}

//
func Register(ctx context.Context, req *pb_v1.RegisterRequest) (reply *pb_v1.RegisterReply, err error) {
	reply = &pb_v1.RegisterReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = checkRegisterParam(req)
	if err != nil {
		return nil, err
	}
	now := time.Now()
	p := &dbmodel.TCompany{
		Applicant:               req.Applicant,
		ApplicantPhone:          req.ApplicantPhone,
		CompanyName:             req.CompanyName,
		CompanyPhone:            req.CompanyPhone,
		ApplicantDuty:           req.ApplicantDuty,
		LegalPerson:             req.LegalPerson,
		CompanyAddr:             req.CompanyAddr,
		SocialCode:              req.SocialCode,
		BusinessLicense:         req.BusinessLicense,
		ApplicantIdentification: req.ApplicantIdentification,
		CreatedAt:               now,
		UpdatedAt:               now,
		ApproveStatus:           consts.ApproveStatusWait,
		LicenseType:             int64(req.LicenseType),
		FreeGardenCount:         int64(req.FreeGardenCount),
		GardenCount:             0,
		User:                    req.Username,
		Password:                "",
		Expire:                  int64(req.Expire),
		UniqFlag:                1,
		Site:                    req.Site,
		Desc:                    req.Desc,
		Logo:                    req.Logo,
	}
	// 运营人员直接添加无需审批
	if req.IsByManager {
		p.ApproveStatus = consts.ApproveStatusSuccess
		p.ApprovedAt = now
	}
	p.Password, _ = util.GetMd5Pass(req.Password, consts.CRYPTO_KEY)

	db := database.DB().Begin()
	err = p.Insert(db)
	if err != nil {
		msg := strings.ToLower(err.Error())
		if strings.Contains(msg, "duplicate") {
			db.Rollback()
			return nil, status.Error(10003, "账号名或社会信用码已存在")
		}
		db.Rollback()
		return nil, errors.DataBaseError
	}
	reply.Id = p.ID

	user := &dbmodel.TUser{
		Phone:     req.ApplicantPhone,
		Cid:       p.ID,
		CreatedAt: now,
		UpdatedAt: now,
		User:      req.Username,
		Password:  p.Password,
		Super:     true,
		RealName:  req.Applicant,
	}

	err = user.Insert(db)
	if err != nil {
		msg := strings.ToLower(err.Error())
		if strings.Contains(msg, "duplicate") {
			db.Rollback()
			return nil, status.Error(10003, "账号名或手机号已存在")
		}
		db.Rollback()
		return nil, errors.DataBaseError
	}
	db.Commit()
	if !req.IsByManager {
		utils.RobotMsg(fmt.Sprintf("物业公司（%s）提交审核，请及时审核", req.CompanyName))
	}
	return reply, nil

}
