// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package statistic

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-company/errors"
	dbmodel "property-company/model"
	pb_v1 "property-company/pb/v1"
	"time"
)

func checkStatisticParam(req *pb_v1.CompanyStatisticRequest) error {

	switch {
	case req.Cid == 0:
		return status.Error(10003, "小区不能为空")
	}
	return nil
}

func CompanyStatistic(ctx context.Context, req *pb_v1.CompanyStatisticRequest) (reply *pb_v1.CompanyStatisticReply, err error) {

	reply = &pb_v1.CompanyStatisticReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	err = checkStatisticParam(req)
	if err != nil {
		return nil, err
	}

	obj := dbmodel.TStatisticObj{}
	where := map[string]interface{}{
		"cid": req.Cid,
	}
	objList, err := obj.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range objList {
		switch v.ObjType {
		case ObjTypeHouse:
			reply.HouseCount = v.Total
		case ObjTypeSpace:
			reply.SpaceCount = v.Total
		case ObjTypeGarden:
			reply.GardenCount = v.Total
		case ObjTypeUser:
			reply.UserCount = v.Total
		}
	}

	now := time.Now()
	nowDay := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
	end := nowDay.Unix()
	start := end - 3600*24*6
	where["date_timestamp >="] = start
	where["date_timestamp <="] = end
	deal := dbmodel.TStatisticDeal{}
	dealList, err := deal.List(database.DB(), where, nil, -1, -1)
	m := map[int64]dbmodel.TStatisticDeal{}
	for _, v := range dealList {
		m[v.DateTimestamp] = v
	}

	reply.DealList = make([]*pb_v1.DealItem, 7)
	for i, _ := range reply.DealList {
		timestamp := start + int64(i*3600*24)
		reply.DealList[i] = &pb_v1.DealItem{
			DealCount:     m[timestamp].DealCount,
			DealAmount:    m[timestamp].DealAmount,
			DateTimestamp: timestamp,
		}

	}

	return reply, err
}
